"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.interval = interval;
var _effector = require("effector");
function interval({
  timeout,
  start,
  stop,
  leading = false,
  trailing = false
}) {
  const setup = start !== null && start !== void 0 ? start : (0, _effector.createEvent)({
    name: "setup",
    sid: "-ajedhm"
  });
  const teardown = stop !== null && stop !== void 0 ? stop : (0, _effector.createEvent)({
    name: "teardown",
    sid: "-ryi8gr"
  });
  const tick = (0, _effector.createEvent)({
    name: "tick",
    sid: "-cw8a02"
  });
  const $isRunning = (0, _effector.createStore)(false, {
    name: "$isRunning",
    sid: "-5gspmy"
  });
  const $timeout = toStoreNumber(timeout);
  const $notRunning = $isRunning.map(running => !running);
  const saveTimeout = (0, _effector.createEvent)({
    name: "saveTimeout",
    sid: "satd6r"
  });
  const $timeoutId = (0, _effector.createStore)(null, {
    name: "$timeoutId",
    sid: "hehxq7"
  }).on(saveTimeout, (_, {
    timeoutId
  }) => timeoutId);
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  const $rejecter = (0, _effector.createStore)(() => {}, {
    name: "$rejecter",
    sid: "-sc3eh9"
  }).on(saveTimeout, (_, {
    reject
  }) => reject);
  const timeoutFx = (0, _effector.attach)({
    and: {
      source: {
        timeout: $timeout,
        running: $isRunning
      },
      effect: ({
        timeout,
        running
      }) => {
        if (!running) {
          return Promise.reject();
        }
        return new Promise((resolve, reject) => {
          const timeoutId = setTimeout(resolve, timeout);
          saveTimeout({
            timeoutId,
            reject
          });
        });
      }
    },
    or: {
      name: "timeoutFx",
      sid: "-xlxric"
    }
  });
  const cleanupFx = (0, _effector.attach)({
    and: {
      source: {
        timeoutId: $timeoutId,
        rejecter: $rejecter
      },
      effect: ({
        timeoutId,
        rejecter
      }) => {
        rejecter();
        if (timeoutId) clearTimeout(timeoutId);
      }
    },
    or: {
      name: "cleanupFx",
      sid: "-32ewxw"
    }
  });
  (0, _effector.guard)({
    and: [{
      clock: setup,
      source: $timeout,
      filter: $notRunning,
      target: timeoutFx
    }],
    or: {
      sid: "-us95i7"
    }
  });
  if (leading) {
    const onReady = (0, _effector.guard)({
      and: [{
        clock: setup,
        filter: $notRunning
      }],
      or: {
        name: "onReady",
        sid: "farho0"
      }
    });
    (0, _effector.sample)({
      and: [{
        clock: onReady,
        target: tick
      }],
      or: {
        sid: "-ubrbnn"
      }
    });
  }
  (0, _effector.sample)({
    and: [{
      clock: setup,
      fn: () => true,
      target: $isRunning
    }],
    or: {
      sid: "-ua3xxa"
    }
  });
  (0, _effector.guard)({
    and: [{
      clock: timeoutFx.done,
      source: $timeout,
      filter: $isRunning,
      target: timeoutFx
    }],
    or: {
      sid: "6lcmx8"
    }
  });
  (0, _effector.guard)({
    and: [{
      clock: timeoutFx.done,
      filter: $isRunning,
      target: tick.prepend(() => {
        /* to be sure, nothing passed to tick */
      })
    }],
    or: {
      sid: "6p772r"
    }
  });
  if (trailing) {
    (0, _effector.sample)({
      and: [{
        clock: teardown,
        target: tick
      }],
      or: {
        sid: "75p0xb"
      }
    });
  }
  $isRunning.on(teardown, () => false);
  (0, _effector.sample)({
    and: [{
      clock: teardown,
      target: cleanupFx
    }],
    or: {
      sid: "7ln232"
    }
  });
  return {
    tick,
    isRunning: $isRunning,
    '@@trigger': () => ({
      setup,
      teardown,
      fired: tick
    })
  };
}
function toStoreNumber(value) {
  if (_effector.is.store(value, {
    sid: "8i2x6t"
  })) return value;
  if (typeof value === 'number') {
    return (0, _effector.createStore)(value, {
      and: {
        name: '$timeout'
      },
      sid: "-jluixv"
    });
  }
  throw new TypeError(`timeout parameter in interval method should be number or Store. "${typeof value}" was passed`);
}

/**
 * @see {@link https://withease.pages.dev/protocols/trigger.html}
 */